package lsedit;

import java.util.Vector;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import java.util.Enumeration;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JViewport;

public class LegendBox extends JComponent implements MouseListener /*, Scrollable */ 
{
	protected static final String g_editInheritanceRules_text  = "Edit inheritance rules";
	protected static final String g_editClassAttributes_text   = "Edit Class Attributes";
	protected static final String g_showValidAttributes_text   = "Show valid attributes";
	protected static final String g_validateAttributes_text    = "Validate attributes";
	protected static final String g_empty_classes              = "Empty Classes";
	protected static final String g_member_counts              = "Member Counts";
	protected static final String g_createEntitiesOfThisClass_text = "Create entities of this class";
	protected static final String g_editRelationConstraints_text   = "Edit relation constraints";
	protected static final String g_closureOfConstraints_text  = "Closure of constraints";
	protected static final String g_validateRelations_text	   = "Validate relations";
	protected static final String g_createEdgesOfThisClass_text = "Create relations of this class";
	protected static final String g_formsHierarchy_text        = "Forms Hierarchy";

	protected static final int MARGIN = 5;
	protected static final int GAP    = 5; 

	private class DisplayClassHierarchy implements ActionListener
	{
		private LegendBox				m_legendBox;
		private LandscapeClassObject	m_landscapeClass;
		int								m_x;
		int								m_y;
		JPopupMenu						m_popup = null;

 
		public DisplayClassHierarchy(LegendBox legendBox, LandscapeClassObject landscapeClass, int x, int y)
		{
			m_legendBox 	 = legendBox;
			m_landscapeClass = landscapeClass;
			m_x				 = x;
			m_y				 = y;
		}

		public void actionPerformed(ActionEvent ev)
		{
			if ((ev.getModifiers() & ActionEvent.META_MASK) != 0) {
				if (m_ls.processMetaKeyEvent(g_editInheritanceRules_text)) {
					return;
			}	}
			LandscapeClassObject	o      = m_landscapeClass;
			ClassInherits			dialog = new ClassInherits(m_ls, o);

			dialog.setLocation(m_x, m_y);
			dialog.setVisible(true);

			Vector	newv   = dialog.getResult();

			if (newv != null) {
				o.updateInherits(newv);
			}
	}	}

	private class EditClassAttributes implements ActionListener
	{
		private EntityClass				m_entity;
 
		public EditClassAttributes(EntityClass entity)
		{
			m_entity    = entity;
		}

		public void actionPerformed(ActionEvent ev)
		{
			if ((ev.getModifiers() & ActionEvent.META_MASK) != 0) {
				if (m_ls.processMetaKeyEvent(g_editClassAttributes_text)) {
					return;
			}	}

			EditAttribute.Create(m_ls, m_entity);
			m_ls.repaint();
	}	}

	private class ShowValidAttributes implements ActionListener
	{
		private LandscapeClassObject m_o;
 
		public ShowValidAttributes(LandscapeClassObject o)
		{
			m_o = o;
		}

		public void actionPerformed(ActionEvent ev)
		{
			if ((ev.getModifiers() & ActionEvent.META_MASK) != 0) {
				if (m_ls.processMetaKeyEvent(g_showValidAttributes_text)) {
					return;
			}	}
			m_ls.showValidAttributes(m_o);
	}	}

	private class CheckEntityAttributes implements ActionListener
	{
		private EntityClass				m_ec;
 
		public CheckEntityAttributes(EntityClass ec)
		{
			m_ec = ec;
		}

		public void actionPerformed(ActionEvent ev)
		{
			if ((ev.getModifiers() & ActionEvent.META_MASK) != 0) {
				if (m_ls.processMetaKeyEvent(g_validateAttributes_text)) {
					return;
			}	}
			m_ls.validateEntityAttributes(m_ec);
	}	}

	private class CheckRelationAttributes implements ActionListener
	{
		private RelationClass			m_rc;
 
		public CheckRelationAttributes(RelationClass rc)
		{
			m_rc = rc;
		}

		public void actionPerformed(ActionEvent ev)
		{
			if ((ev.getModifiers() & ActionEvent.META_MASK) != 0) {
				if (m_ls.processMetaKeyEvent(g_validateAttributes_text)) {
					return;
			}	}
			m_ls.validateRelationAttributes(m_rc);
	}	}

	private class CheckRelations implements ActionListener
	{
		private RelationClass			m_rc;
 
		public CheckRelations(RelationClass rc)
		{
			m_rc = rc;
		}

		public void actionPerformed(ActionEvent ev)
		{
			if ((ev.getModifiers() & ActionEvent.META_MASK) != 0) {
				if (m_ls.processMetaKeyEvent(g_validateRelations_text)) {
					return;
			}	}
			m_ls.validateRelationAttributes(m_rc);
			m_ls.validateRelations(m_rc);
	}	}


	private class EditRelationClassAttributes implements ActionListener
	{
		private LandscapeEditorCore		m_ls;
		private RelationClass			m_relation;
 
		public EditRelationClassAttributes(LandscapeEditorCore ls, RelationClass relation)
		{
			m_ls	   = ls;
			m_relation = relation;
		}

		public void actionPerformed(ActionEvent ev)
		{
			if ((ev.getModifiers() & ActionEvent.META_MASK) != 0) {
				if (m_ls.processMetaKeyEvent(g_editClassAttributes_text)) {
					return;
			}	}
			EditAttribute.Create(m_ls, m_relation);
			m_ls.repaint();
	}	}

	private class DeleteEntityClass implements ActionListener
	{
		private EntityClass	m_ec;
 
		public DeleteEntityClass(EntityClass ec)
		{
			m_ec      = ec;
		}

		public void actionPerformed(ActionEvent ev)
		{
			EntityClass ec      = m_ec;
			Diagram		diagram = m_ls.getDiagram();
			EntityInstance rootInstance;
			Enumeration en;
			EntityClass	ec1;
			String		message = null;
			int			cnt;

			if ((ev.getModifiers() & ActionEvent.META_MASK) != 0) {
				if (m_ls.processMetaKeyEvent("Delete Entity Class")) {
					return;
			}	}

			for (en = diagram.enumEntityClasses(); en.hasMoreElements(); ) {
				ec1 = (EntityClass) en.nextElement();
				if (ec != ec1 && ec1.directlyInheritsFrom(ec)) {
					if (message != null) {
						message += ", ";
					} else {
						message = "";
					}
					message += ec1.getLabel();
			}	}
			if (message != null) {
				message += " inherit from " + ec.getLabel();
			} else {
				rootInstance = diagram.getRootInstance();
				if (rootInstance != null && rootInstance.getEntityClass() == ec) {
					message = "Root instance has class " + ec.getLabel();
				} else {
					diagram.recomputeCounts();
					cnt = ec.countMembers();

					if (cnt != 0) {
						message = ec.getLabel() + " has " + cnt + " instantiations";
			}	}	}
		
			if (message != null) {
				JOptionPane.showConfirmDialog(null, message, "Can't delete entity class", JOptionPane.DEFAULT_OPTION);
				return;
			}

			switch (JOptionPane.showConfirmDialog(null, "Delete class " + ec.getLabel(), "Delete all " + ec.getLabel() + " entities", JOptionPane.YES_NO_CANCEL_OPTION)) {
			case JOptionPane.YES_OPTION:
				m_ls.beginUndoRedo("Delete " + ec.getLabel() + " entity class");
				diagram.updateRemoveEntityClass(ec);
				diagram.clearGroupFlags();
				m_ls.endUndoRedo();

				diagram.redrawDiagram();
				break;
			case JOptionPane.NO_OPTION:
				break;
			}
			m_ls.repaint();
	}	}

	private class DeleteRelationClass implements ActionListener
	{
		private RelationClass	m_rc;
 
		public DeleteRelationClass(RelationClass rc)
		{
			m_rc      = rc;
		}

		public void actionPerformed(ActionEvent ev)
		{
			if ((ev.getModifiers() & ActionEvent.META_MASK) != 0) {
				if (m_ls.processMetaKeyEvent("Delete Relation Class")) {
					return;
			}	}


			RelationClass	rc      = m_rc;
			Diagram			diagram = m_ls.getDiagram();
			Enumeration		en;
			RelationClass	rc1;
			int				cnt;
			String			message = null;


			for (en = diagram.enumRelationClasses(); en.hasMoreElements(); ) {
				rc1 = (RelationClass) en.nextElement();
				if (rc != rc1 && rc1.directlyInheritsFrom(rc)) {
					if (message != null) {
						message += ", ";
					} else {
						message  = "";
					}
					message += rc1.getLabel();
			}	}
			if (message != null) {
				message += " inherit from " + rc.getLabel();
			} else {
				diagram.recomputeCounts();
				cnt = rc.countMembers();
				if (cnt != 0) {
					message = rc.getLabel() + " has " + cnt + " instantiations";
			}	}

			if (message != null) {
				JOptionPane.showConfirmDialog(null, message, "Can't delete relation class", JOptionPane.DEFAULT_OPTION);
				return;
			}

			switch (JOptionPane.showConfirmDialog(null, "Delete class " + rc.getLabel(), "Delete all " + rc.getLabel() + " edges", JOptionPane.YES_NO_CANCEL_OPTION)) {
			case JOptionPane.YES_OPTION:
				m_ls.beginUndoRedo("Delete " + rc.getLabel() + " relation class");
				diagram.updateRemoveRelationClass(rc);
				m_ls.endUndoRedo();
				diagram.redrawDiagram();
				break;
			case JOptionPane.NO_OPTION:
				return;
			default:
				return;
			}
	}	}

	private class ShowConstraintsMatrix implements ActionListener
	{
		private Diagram			m_diagram;
		private RelationClass	m_rc;
 
		public ShowConstraintsMatrix(Diagram diagram, RelationClass rc)
		{
			m_diagram = diagram;
			m_rc      = rc;
		}

		public void actionPerformed(ActionEvent ev)
		{
			if ((ev.getModifiers() & ActionEvent.META_MASK) != 0) {
				if (m_ls.processMetaKeyEvent(g_editRelationConstraints_text)) {
					return;
			}	}

			EditConstraints.create(m_diagram, m_rc);
	}	}

	private class ShowConstraintsClosure implements ActionListener
	{
		private Diagram			m_diagram;
		private RelationClass	m_rc;
 
		public ShowConstraintsClosure(Diagram diagram, RelationClass rc)
		{
			m_diagram = diagram;
			m_rc      = rc;
		}

		public void actionPerformed(ActionEvent ev)
		{
			if ((ev.getModifiers() & ActionEvent.META_MASK) != 0) {
				if (m_ls.processMetaKeyEvent(g_closureOfConstraints_text)) {
					return;
			}	}

			ClosureConstraints.create(m_diagram, m_rc);
	}	}


	private class SetDefaultEntityClass implements ActionListener
	{
		private Diagram		m_diagram;
		private EntityClass	m_ec;
 
		public SetDefaultEntityClass(Diagram diagram, EntityClass ec)
		{
			m_diagram = diagram;
			m_ec      = ec;
		}

		public void actionPerformed(ActionEvent ev)
		{
			if ((ev.getModifiers() & ActionEvent.META_MASK) != 0) {
				if (m_ls.processMetaKeyEvent(g_createEntitiesOfThisClass_text)) {
					return;
			}	}

			m_diagram.setDefaultEntityClass(m_ec);
	}	}

	private class SetDefaultRelationClass implements ActionListener
	{
		private Diagram			m_diagram;
		private RelationClass	m_rc;
 
		public SetDefaultRelationClass(Diagram diagram, RelationClass rc)
		{
			m_diagram = diagram;
			m_rc      = rc;
		}

		public void actionPerformed(ActionEvent ev)
		{
			if ((ev.getModifiers() & ActionEvent.META_MASK) != 0) {
				if (m_ls.processMetaKeyEvent(g_createEdgesOfThisClass_text)) {
					return;
			}	}

			m_diagram.setDefaultRelationClass(m_rc);
	}	}


	private class SetContainsRelation implements ActionListener
	{
		private LandscapeEditorCore		m_ls;
		private RelationClass			m_relation;
 
		public SetContainsRelation(LandscapeEditorCore ls, RelationClass relation)
		{
			m_ls	   = ls;
			m_relation = relation;
		}

		public void actionPerformed(ActionEvent ev)
		{
			if ((ev.getModifiers() & ActionEvent.META_MASK) != 0) {
				if (m_ls.processMetaKeyEvent(g_formsHierarchy_text)) {
					return;
			}	}
			m_ls.updateContainsRelation(m_relation);
		}
	}

	private class ActiveEntityChkBox extends EntityChkBox implements ItemListener, MouseListener
	{
		protected	JPopupMenu			m_popup = null;

		public ActiveEntityChkBox(EntityClass ec, int index, int count, int h) 
		{
			super(ec, index, count, h, false);

			setFont(m_textFont);
			addItemListener(this);
			addMouseListener(this);
		}

		public boolean isActive()
		{
			return m_ec.isShown();
		}

		public void setActive(boolean value)
		{
			if (m_ec.isShown() != value) {
				m_ec.setShown(value);
				m_ls.getDiagram().redrawDiagram();
		}	}

		public void itemStateChanged(ItemEvent ev)
		{
//			System.out.println("ActiveEntityChkBox.itemStateChanged " + ev.getStateChange() + " " + isSelected());
			setActive(ev.getStateChange() == ItemEvent.SELECTED);
		}


		// MouseListener interface

		public void mouseClicked(MouseEvent e)
		{
		}

		public void mouseEntered(MouseEvent e)
		{
		}

		public void mouseExited(MouseEvent e)
		{
		}

		public void mousePressed(MouseEvent ev)
		{
			if (ev.isMetaDown()) {
				int					x  = ev.getX();
				int					y  = ev.getY();

				if (m_popup == null) {
					JMenuItem	mi;
					Diagram		diagram = m_ls.getDiagram();

					m_popup = new JPopupMenu("Class menu");

					mi = new JMenuItem(g_editInheritanceRules_text);
					mi.addActionListener(new DisplayClassHierarchy(LegendBox.this, m_ec, x, y)); 
					m_popup.add(mi);

					mi = new JMenuItem(g_editClassAttributes_text);
					mi.addActionListener(new EditClassAttributes(m_ec)); 
					m_popup.add(mi);

					mi = new JMenuItem(g_showValidAttributes_text);
					mi.addActionListener(new ShowValidAttributes(m_ec)); 
					m_popup.add(mi);

					mi = new JMenuItem(g_validateAttributes_text);
					mi.addActionListener(new CheckEntityAttributes(m_ec)); 
					m_popup.add(mi);

					mi = new JMenuItem(g_createEntitiesOfThisClass_text);
					mi.addActionListener(new SetDefaultEntityClass(diagram, m_ec));
					m_popup.add(mi);

					if (m_ec != diagram.m_entityBaseClass) {
						mi = new JMenuItem("Delete entity class " + m_ec.getLabel());
						mi.addActionListener(new DeleteEntityClass(m_ec));
						m_popup.add(mi);
					}
					LegendBox.this.add(m_popup);
				}
				FontCache.setMenuTreeFont(m_popup); 

//				Do.dump_menu(m_popup);
				m_popup.show(this, x, y);
		}	}

		public void mouseReleased(MouseEvent ev) 
		{
		}
	}
		
	class ActiveRelnChkBox extends RelnChkBox /* extends JComponent */ implements ItemListener, MouseListener
	{
		protected	JPopupMenu			m_popup;

		public ActiveRelnChkBox(RelationClass rc,  int index, int count, Font font) 
		{
			super(rc, index, count, rc.isClassShown(), font); 

			addItemListener(this);
			addMouseListener(this);
		}

		public boolean isContainsClass()
		{
			return (m_rc.isContainsClass());
		}

		public boolean isActive()
		{
			return(m_rc.isClassShown());
		}

		public void setActive(boolean value)
		{
			if (isActive() != value) {
				m_rc.setShown(value);
				m_ls.getDiagram().redrawDiagram();
		}	}

		public void itemStateChanged(ItemEvent ev)
		{
//			System.out.println("ActiveRelnChkBox.itemStateChanged " + ev.getStateChange() + " " + isSelected());
			setActive(ev.getStateChange() == ItemEvent.SELECTED);
		}

		// MouseListener interface

		public void mouseClicked(MouseEvent e)
		{
		}

		public void mouseEntered(MouseEvent e)
		{
		}

		public void mouseExited(MouseEvent e)
		{
		}

		public void mousePressed(MouseEvent ev)
		{
			if (ev.isMetaDown()) {

				int		x           = ev.getX();
				int		y           = ev.getY();

				JMenuItem	mi;
				Diagram		diagram = m_ls.getDiagram();

				m_popup = new JPopupMenu("Relation menu");

				mi = new JMenuItem(g_editInheritanceRules_text);
				mi.addActionListener(new DisplayClassHierarchy(LegendBox.this, m_rc, x, y)); 
				m_popup.add(mi);

				mi = new JMenuItem(g_editClassAttributes_text);
				mi.addActionListener(new EditRelationClassAttributes(m_ls, (RelationClass) m_rc)); 
				m_popup.add(mi);

				mi = new JMenuItem(g_editRelationConstraints_text);
				mi.addActionListener(new ShowConstraintsMatrix(diagram, (RelationClass) m_rc));
				m_popup.add(mi);

				mi = new JMenuItem(g_closureOfConstraints_text);
				mi.addActionListener(new ShowConstraintsClosure(diagram, (RelationClass) m_rc));
				m_popup.add(mi);

				mi = new JMenuItem(g_showValidAttributes_text);
				mi.addActionListener(new ShowValidAttributes(m_rc)); 
				m_popup.add(mi);

				mi = new JMenuItem(g_validateAttributes_text);
				mi.addActionListener(new CheckRelationAttributes(m_rc)); 
				m_popup.add(mi);

				mi = new JMenuItem(g_validateRelations_text);
				mi.addActionListener(new CheckRelations(m_rc)); 
				m_popup.add(mi);

				if (!m_rc.isContainsClass()) {

					if (m_rc != diagram.m_relationBaseClass) {
						mi = new JMenuItem("Delete relation class " + m_rc.getLabel());
						mi.addActionListener(new DeleteRelationClass(m_rc));
						m_popup.add(mi);
					}

					mi = new JMenuItem(g_createEdgesOfThisClass_text);
					mi.addActionListener(new SetDefaultRelationClass(diagram, m_rc));
					m_popup.add(mi);

					mi = new JMenuItem(g_formsHierarchy_text);
					mi.addActionListener(new SetContainsRelation(m_ls, (RelationClass) m_rc)); 
					m_popup.add(mi);
				}

				FontCache.setMenuTreeFont(m_popup); 
				LegendBox.this.add(m_popup);
//				Do.dump_menu(m_popup);
				m_popup.show(this, x, y);
		}	}
		
		public void mouseReleased(MouseEvent ev) 
		{
		}
	}

	protected LandscapeEditorCore   m_ls;
	protected JTabbedPane			m_tabbedPane;
	protected JScrollPane			m_scrollPane;

	public final static String DEFAULT_LEGEND_TITLE_FONT_NAME  = FontCache.DEFAULT_FONT_NAME;
	public final static int    DEFAULT_LEGEND_TITLE_FONT_STYLE = Font.BOLD;
	public final static int    DEFAULT_LEGEND_TITLE_FONT_SIZE  = 12;

	protected static Font	m_titleFont  = FontCache.get(DEFAULT_LEGEND_TITLE_FONT_NAME, DEFAULT_LEGEND_TITLE_FONT_STYLE, DEFAULT_LEGEND_TITLE_FONT_SIZE);

	public final static String DEFAULT_LEGEND_TEXT_FONT_NAME  = FontCache.DEFAULT_FONT_NAME;
	public final static int    DEFAULT_LEGEND_TEXT_FONT_STYLE = Font.PLAIN;
	public final static int    DEFAULT_LEGEND_TEXT_FONT_SIZE  = 11;

	protected static Font   m_textFont	 = FontCache.get(DEFAULT_LEGEND_TEXT_FONT_NAME, DEFAULT_LEGEND_TEXT_FONT_STYLE, DEFAULT_LEGEND_TEXT_FONT_SIZE);

	public    static final String m_helpStr	 = "This box shows the types of entities and relations that are present in the current landscape.";

	protected JLabel				m_ulabel;	// TODO underline it
	protected JLabel				m_elabel1;
	protected JLabel				m_elabel2;
	protected JLabel				m_rlabel;	// TODO underline it
	protected JLabel				m_clabel;
	protected JLabel				m_flabel1;
	protected JLabel				m_flabel2;

	protected int					m_width;
	protected int					m_height;


	// --------------
	// Public methods 
	// --------------

	public LegendBox(LandscapeEditorCore ls, JTabbedPane tabbedPane) 
	{
		m_ls          = ls;
		m_tabbedPane  = tabbedPane;

		m_ulabel = new JLabel("Entity Classes");
		m_ulabel.setForeground(Color.red);
		m_ulabel.setFont(m_titleFont);

		m_elabel1 = new JLabel("Pass cursor over entity for description.");
		m_elabel1.setForeground(Color.black);
		m_elabel1.setFont(m_textFont);

		m_elabel2 = new JLabel("Right click for menu.");
		m_elabel2.setForeground(Color.black);
		m_elabel2.setFont(m_textFont);

		m_rlabel = new JLabel("Relation Classes");
		m_rlabel.setForeground(Color.red);
		m_rlabel.setFont(m_titleFont);

		m_clabel = new JLabel("Hierarchy");
		m_clabel.setForeground(Color.red);
		m_clabel.setFont(m_titleFont);

		m_flabel1 = new JLabel("Checkboxes select visible relations.");
		m_flabel1.setForeground(Color.black);
		m_flabel1.setFont(m_textFont);

		m_flabel2 = new JLabel("Right click for menu.");
		m_flabel2.setForeground(Color.black);
		m_flabel2.setFont(m_textFont);


		setLayout(null);
		setBackground(Diagram.boxColor);
		setToolTipText(m_helpStr);

		m_scrollPane = new JScrollPane();
		m_scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED); 
		m_scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED); 

		m_scrollPane.setViewportView(this);

		tabbedPane.addTab("Legend", null, m_scrollPane, m_helpStr);
		addMouseListener(this);
	}

	public static Font getTitleFont()
	{
		return m_titleFont;
	}

	public static void setTitleFont(Font font)
	{
		m_titleFont = font;
	}

	public void titleFontChanged(Font font)
	{
		m_ulabel.setFont(font);
		m_rlabel.setFont(font);
		fillLegendBox();
	}

	public static Font getTextFont()
	{
		return m_textFont;
	}

	public static void setTextFont(Font font)
	{
		m_textFont = font;
	}

	public void textFontChanged(Font font)
	{
		m_elabel1.setFont(font);
		m_elabel2.setFont(font);
		m_flabel1.setFont(font);
		m_flabel2.setFont(font);
		fillLegendBox();
	}

	public void activate() 
	{
		m_tabbedPane.setSelectedComponent(m_scrollPane);
	}

	public boolean isActive() 
	{
		Component active;

		if (!isVisible()) {
			return(false);
		}
		active = m_tabbedPane.getSelectedComponent();
		return(active != null && m_scrollPane == active);
	}

	public void containsClassChanged()
	{
		fillLegendBox();
	}

/*
	public void setBounds(int x, int y, int width, int height)
	{
		super.setBounds(x,y,width,height);
		System.out.println("Legend size " + getBounds());
		return;
	}
*/

	protected void add(JComponent component)
	{
		Dimension	d;
		int			width, height;

		d      = component.getPreferredSize();
		width  = d.width;
		height = d.height;
		if (width > m_width) {
//			System.out.println("Width=" + width + " for " + component);
			m_width = width;
		}
		component.setBounds(MARGIN, m_height, width, height);
		super.add(component);
		m_height += height;
	}

	public void fillLegendBox()
	{
		Diagram				diagram;
		FontMetrics			fontMetrics;
		Enumeration			en;
		int					item,n,h;
		JViewport			viewport;
		int					width, height;
		Dimension 			dimension;
		RelationClass		contains;
		int					cnt;
		boolean				isHideEmpty    = m_ls.isHideEmpty();
		boolean				isMemberCounts = m_ls.isMemberCounts();
		boolean				usesCounts     = isHideEmpty || isMemberCounts;

/*		System.out.println("LegendBox.fillLegendBox()");
		java.lang.Thread.dumpStack();
		System.out.println("-----");
*/

		diagram = m_ls.getDiagram();

		if (usesCounts) {
			diagram.recomputeCounts();
		}
		removeAll();

		m_width  = 0;
		m_height = 0;

		// Draw legend for entities

		item = 0;
		n	 = 0;

		fontMetrics = getFontMetrics(m_textFont);
		h           = fontMetrics.getHeight();

		EntityClass ec;

		m_height += 10;
		add(m_ulabel);
		cnt       = -1;

		if (diagram != null) {
			for (en = diagram.enumEntityClassesInOrder(); en.hasMoreElements(); ) {
				ec = (EntityClass) en.nextElement();

				if (usesCounts) {
					cnt = ec.countMembers();
					if (isHideEmpty && cnt == 0) {
						continue;
					}
					if (!isMemberCounts) {
						cnt = -1;
				}	}
				m_height += 10;
				add(new ActiveEntityChkBox(ec, ++n, cnt, h));
		}	}

		m_height += 10;
		add(m_elabel1);
		add(m_elabel2);
		m_height += 20;
		add(m_rlabel);

		// Draw legend for relations

		m_height += GAP;

		n = 0;
		contains = null;
		if (diagram != null) {
			for (en = diagram.enumRelationClassesInOrder(); en.hasMoreElements(); ) {
				RelationClass rc = (RelationClass) en.nextElement();

				if (rc.isContainsClass()) {
					contains = rc;
				} else {
					if (usesCounts) {
						cnt = rc.countMembers();
						if (isHideEmpty && cnt == 0) {
							continue;
						}
						if (!isMemberCounts) {
							cnt = -1;
					}	}
					add(new ActiveRelnChkBox(rc, ++n, cnt, m_textFont));
		}	}	}

		if (contains != null) {
			m_height += 10;
			add(m_clabel);
			m_height += GAP;
			add(new ActiveRelnChkBox(contains, -1, -1, m_textFont));
		}

		m_height += 20;
		add(m_flabel1);
		add(m_flabel2);

		// Max width is computed in the add() method
		m_width  += MARGIN;
		m_height += 10;

		dimension = new Dimension(m_width, m_height);
		setPreferredSize(dimension);
		setBounds(0, 0, m_width, m_height);
	}

/*
	public Dimension getMinimumSize()
	{
		return(getPreferredSize());
	}
*/
	
	public Dimension getMaximumSize()
	{
		return(getPreferredSize());
	}

	public void toggleRelationVisibility(int key) 
	{
		int				cnt = getComponentCount();
		int				i;
		Component		c;
		boolean			ns;

		ns = true;
		if (key == 0) {
			for (i = 0; i < cnt; ++i) {
				c = getComponent(i);
				if (c instanceof ActiveRelnChkBox) {
					if ( ((ActiveRelnChkBox) c).getIndex() == 1) {
						// Change everything the same way as the way the first one changed
						ns = !((ActiveRelnChkBox) c).isActive();
						break;
			}	}	}

			for (i = 0; i < cnt; ++i) {
				c = getComponent(i);
				if (c instanceof ActiveRelnChkBox) {
					if (ns != ((ActiveRelnChkBox) c).isActive()) {
						((ActiveRelnChkBox) c).doClick();
			}	}	}
		} else {
			for (i = 0; i < cnt; ++i) {
				c = getComponent(i);
				if (c instanceof ActiveRelnChkBox) {
					if (((ActiveRelnChkBox) c).getIndex() == key) {
						((ActiveRelnChkBox) c).doClick();
						break;
		}	}	}	}
	}

	private class SetEntitiesShown implements ActionListener
	{
		boolean	m_active;

		public SetEntitiesShown(boolean active)
		{
			m_active = active;
		}

		public void actionPerformed(ActionEvent ev)
		{
			if ((ev.getModifiers() & ActionEvent.META_MASK) != 0) {
				if (m_ls.processMetaKeyEvent("Set Entities Shown")) {
					return;
			}	}

			int				cnt    = getComponentCount();
			boolean			active = m_active;
			int				i;
			Component		c;
			ActiveEntityChkBox	box;

			for (i = 0; i < cnt; ++i) {
				c = getComponent(i);
				if (c instanceof ActiveEntityChkBox) {
					box = (ActiveEntityChkBox) c;
					if (box.isActive() != active) {
						box.doClick();
		}	}	}	}	
	}

	private class SetRelationsShown implements ActionListener
	{
		boolean	m_active;

		public SetRelationsShown(boolean active)
		{
			m_active = active;
		}

		public void actionPerformed(ActionEvent ev)
		{
			if ((ev.getModifiers() & ActionEvent.META_MASK) != 0) {
				if (m_ls.processMetaKeyEvent("Set Relations Shown")) {
					return;
			}	}

			int				cnt    = getComponentCount();
			boolean			active = m_active;
			int				i;
			Component		c;
			ActiveRelnChkBox	box;

			for (i = 0; i < cnt; ++i) {
				c = getComponent(i);
				if (c instanceof ActiveRelnChkBox) {
					box = (ActiveRelnChkBox) c;
					if (!box.isContainsClass() && box.isActive() != active) {
						box.doClick();
		}	}	}	}	
	}

	class ToggleEmptyClasses implements ActionListener
	{
		public ToggleEmptyClasses()
		{
		}

		public void actionPerformed(ActionEvent ev)
		{
			if ((ev.getModifiers() & ActionEvent.META_MASK) != 0) {
				if (m_ls.processMetaKeyEvent(g_empty_classes)) {
					return;
			}	}
			m_ls.setHideEmpty(!m_ls.isHideEmpty());
			fillLegendBox();
	}	}

	class ToggleMemberCounts implements ActionListener
	{
		public ToggleMemberCounts()
		{
		}

		public void actionPerformed(ActionEvent ev)
		{
			if ((ev.getModifiers() & ActionEvent.META_MASK) != 0) {
				if (m_ls.processMetaKeyEvent(g_member_counts)) {
					return;
			}	}
			m_ls.setMemberCounts(!m_ls.isMemberCounts());
			fillLegendBox();
	}	}

	// Generic 	MouseListener interface

	protected void doRightPopup(MouseEvent ev)
	{
		LandscapeEditorCore	ls  = m_ls;
		Diagram			diagram = ls.getDiagram();
		Enumeration		en;
		EntityClass		ec;
		RelationClass	rc;
		int				x, y;
		JPopupMenu		popupMenu;
		JMenuItem		mi;
		String			string;
				
		x         = ev.getX();
		y         = ev.getY();
		popupMenu = new JPopupMenu("Legend options");

		for (en = diagram.enumEntityClasses(); en.hasMoreElements(); ) {
			ec = (EntityClass) en.nextElement();
			if (ec.isShown()) {
				mi = new JMenuItem("No entities shown");
				mi.addActionListener(new SetEntitiesShown(false));
				popupMenu.add(mi);
				break;
		}	}

		for (en = diagram.enumEntityClasses(); en.hasMoreElements(); ) {
			ec = (EntityClass) en.nextElement();
			if (!ec.isShown()) {				
				mi = new JMenuItem("All entities shown");
				mi.addActionListener(new SetEntitiesShown(true));
				popupMenu.add(mi);
				break;
		}	}
	
		for (en = diagram.enumRelationClasses(); en.hasMoreElements(); ) {
			rc = (RelationClass) en.nextElement();
			if (!rc.isContainsClass() && rc.isShown()) {
				mi = new JMenuItem("No relations shown");
				mi.addActionListener(new SetRelationsShown(false));
				popupMenu.add(mi);
				break;
		}	}

		for (en = diagram.enumRelationClasses(); en.hasMoreElements(); ) {
			rc = (RelationClass) en.nextElement();
			if (!rc.isContainsClass() && !rc.isShown()) {
				mi = new JMenuItem("All relations shown");
				mi.addActionListener(new SetRelationsShown(true));
				popupMenu.add(mi);
				break;
		}	}

		Do.createClassMenuItem(popupMenu, ls);

		if (ls.isHideEmpty()) {
			string = "Show ";
		} else {
			string = "Hide ";
		}

		mi = new JMenuItem(string + g_empty_classes);
		mi.addActionListener(new ToggleEmptyClasses());
		popupMenu.add(mi);

		if (ls.isMemberCounts()) {
			string = "Hide ";
		} else {
			string = "Show ";
		}

		mi = new JMenuItem(string + g_member_counts);
		mi.addActionListener(new ToggleMemberCounts());
		popupMenu.add(mi);

		FontCache.setMenuTreeFont(popupMenu); 
		add(popupMenu);
		popupMenu.show(this, x, y);
//		Do.dump_menu(popupMenu);
		remove(popupMenu);
	}

	public void mouseClicked(MouseEvent ev)
	{
		
		if (ev.isMetaDown()) {
			doRightPopup(ev);
	}	}

	public void mouseEntered(MouseEvent ev)
	{
	}

	public void mouseExited(MouseEvent e)
	{
	}

	public void mousePressed(MouseEvent ev)
	{
	}

	public void mouseReleased(MouseEvent ev)
	{
	}

/*
	// Scrollable interface
	// We need to implement this to stop iterative size changes..
	// The mapbox knows what size it wants to be and sets it on every validate
	// If we don't disable it the viewport tries to change the size to fit the window every time it validates

	public Dimension getPreferredScrollableViewportSize()
	{
		return(getSize());
	}
	 
	public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction)
	{
		return(50);
	}
	 
	public boolean getScrollableTracksViewportHeight() 
	{
		return(false);
	}

	public boolean getScrollableTracksViewportWidth() 
	{
		return(false);
	}

	public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction)
	{
		return(10);
	} 
*/
}



